require_relative './api_credentials'
require 'httparty'
require 'json'

module Gugl
  module Search
    class RemoteSearcher
      attr_accessor :credentials
      
      ITEMS_RESPONSE_NODE = 'items'
      ERROR_RESPONSE_NODE = 'error'
      
      def search(query, limit)
        data = fetch_and_parse(query, limit)
        if results = data[ITEMS_RESPONSE_NODE]
          yield results
        else
          raise "Error: #{data[ERROR_RESPONSE_NODE]}"
        end
      end
      
      def authorize
        self.credentials = ApiCredentials.new
        yield credentials
      end
  
      private
      
      def fetch_and_parse(query, limit)
        response = fetch(query, limit)
        parse_response(response)
      end
      
      def fetch(query, limit)
        HTTParty.get(base_url, query: {
          num: limit,
          q: query
        }.merge(credentials.to_h))
      end
  
      def base_url
        "https://www.googleapis.com/customsearch/v1"
      end
      
      def parse_response(response)
        JSON.parse(response.body)
      end
    end
  end
end