require_relative './data_getter'
require_relative './displayers/displayer_mapper'

module Gugl
  module Search
    class Searcher
      attr_reader :results
      attr_accessor :limit, :query
  
      DEFAULT_LIMIT = 5
  
      def search
        configure_getter
        @results = getter.get
      end
  
      def display
        results.each_with_index.map do |result, index|
          displayer.display result, index
        end
      end
      
      def authorize(&blk)
        getter.searcher.authorize(&blk)
      end
      
      def search_and_display
        search
        display
      end
      
      def displayer=(displayer)
        @displayer = if displayer.is_a?(Symbol)
          mapped_displayer(displayer)
        else
          displayer
        end
      end
      
      def displayer
        @displayer ||= mapped_displayer(:default)
      end
      
      def limit
        @limit ||= DEFAULT_LIMIT
      end
  
      private
      
      def mapped_displayer(key)
        Displayers::DisplayerMapper[key]
      end
      
      def getter
        @getter ||= DataGetter.new
      end
  
      def configure_getter
        getter.query = query
        getter.limit = limit
      end
      
    end
  end
end